import java.time.LocalDate;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.function.Predicate;

public class Checkbook {
    private ArrayList<Transaction> transactionList;
    private GenericAccount account;
    private Payee owner;

    Checkbook(double cashAmount){
        this.transactionList = new ArrayList<>();
        this.account = new GenericAccount();
    }
    Checkbook(Payee owner, double cashAmount){
        this.owner = owner;
        this.account = new GenericAccount(cashAmount);
        this.transactionList = new ArrayList<>();
    }

    public int size(){
        return transactionList.size();
    }

    public boolean isEmpty(){
        return transactionList.isEmpty();
    }

    public boolean contains(Transaction element){
        return transactionList.contains(element);
    }

    public boolean add(Transaction element){
        if (element instanceof Transaction){
            transactionList.add(element);
            account.setBalance(element.getAmount());
            return true;
        }
        return false;
    }

    public boolean remove(Transaction element){
        if (element instanceof Transaction && transactionList.contains(element)){
            account.setBalance(element.getAmount() * -1);
            transactionList.remove(element);
            return true;
        }
        return false;
    }

    public Iterator<Transaction> iterator(){
        return transactionList.iterator();
    }

    public Transaction[] toArray(){
        return transactionList.toArray(new Transaction[transactionList.size()] );
    }

    public Transaction[] toArray(Transaction[] a){
        return a;
    }

    public double checkbookValue(){ // sum of all Transactions (both Debits and Credits)
        return account.getBalance();
    }

    public Transaction[] getTransactionsForDate(LocalDate aDate){
        Predicate<Transaction> predicate = t -> t.getDate().equals(aDate);
        return transactionListFilter(predicate);
    }

    public Transaction[] getAllTransactionsForPayee(Payee payee){
        Predicate<Transaction> predicate = t -> t.getPayee().equals(payee);
        return transactionListFilter(predicate);
    }

    public Transaction[] getAllDebitTransactions(){
        Predicate<Transaction> predicate = t -> t.getType() == TransactionType.DEBIT;
        return transactionListFilter(predicate);
    }

    public Transaction[] getAllCreditTransactions(){
        Predicate<Transaction> predicate = t -> t.getType() == TransactionType.CREDIT;
        return transactionListFilter(predicate);
    }

    private Transaction[] transactionListFilter(Predicate<Transaction> predicate){
        return Arrays.stream(transactionList.toArray(new Transaction[transactionList.size()]))
                .filter(predicate)
                .toArray(Transaction[]::new);
    }


}


//    public Transaction getTransactionsForDate(LocalDate aDate){
//        Predicate<Transaction> predicate = t -> t.getDate().equals(aDate);
//        return Arrays.stream(transactionList.toArray(new Transaction[transactionList.size()]))
//                .filter(t -> t.getDate().equals(aDate)) //predicate
//                .findFirst()
//                .get();
//    }
//
//    public Transaction[] getAllTransactionsForPayee(Payee payee){
//        return Arrays.stream(transactionList.toArray(new Transaction[transactionList.size()]))
//                .filter(t -> t.getPayee().equals(payee)) //predicate
//                .toArray(Transaction[]::new);
//    }
//
//    public Transaction[] getAllDebitTransactions(){
//        return Arrays.stream(transactionList.toArray(new Transaction[transactionList.size()]))
//                .filter(t -> t.getType() == TransactionType.DEBIT) //predicate
//                .toArray(Transaction[]::new);
//    }
//
//    public Transaction[] getAllCreditTransactions(){
//        return Arrays.stream(transactionList.toArray(new Transaction[transactionList.size()]))
//                .filter(t -> t.getType() == TransactionType.CREDIT) //predicate
//                .toArray(Transaction[]::new);
//    }
//
//    public Transaction[] test(Predicate<Transaction> predicate){
//        return Arrays.stream(transactionList.toArray(new Transaction[transactionList.size()]))
//                .filter(predicate) //predicate
//                .toArray(Transaction[]::new);
