import junit.framework.Assert;
import org.junit.Test;

import javax.swing.text.html.HTMLDocument;

import java.time.LocalDate;
import java.util.stream.Stream;

import static org.junit.Assert.*;

public class CheckbookTest {
    Checkbook checkbook;
    Checkbook checkbookNoOwner;
    Transaction debit;
    Transaction credit;
    Payee payeeDebit;
    Payee payeeCredit;
    Payee owner;

    @org.junit.Before
    public void setUp() throws Exception {
        this.owner = new Payee();
             owner.setGivenName("Nick Satinover");
        this.payeeCredit = new Payee();
             payeeCredit.setGivenName("Credit Payee");
        this.payeeDebit = new Payee();
             payeeDebit.setGivenName("Debit Payee");
        this.debit = new Transaction("This is a Debit", payeeDebit, TransactionType.DEBIT, 100.50);
        this.credit = new Transaction("This is a Credit", payeeCredit, TransactionType.CREDIT, 100.50);
        this.checkbook = new Checkbook(owner, 1000.00);

        this.checkbookNoOwner = new Checkbook(100.50);
    }

    @Test
    public void test1Size(){
        checkbook.add(debit);
        int expected = 1;
        int actual = checkbook.size();
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void test2Size(){
        checkbook.add(debit);
        checkbook.add(credit);
        int expected = 2;
        int actual = checkbook.size();
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void test3Size(){
        checkbook.add(debit);
        checkbook.add(debit);
        checkbook.add(debit);
        checkbook.remove(debit);
        int expected = 2;
        int actual = checkbook.size();
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void test1IsEmpty(){
        checkbook.add(debit);
        boolean expected = false;
        boolean actual = checkbook.isEmpty();
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void test2IsEmpty(){
        checkbook.add(debit);
        boolean expected = false;
        boolean actual = checkbook.isEmpty();
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void test3IsEmpty(){
        boolean expected = true;
        boolean actual = checkbook.isEmpty();
        Assert.assertEquals(expected, actual);
    }
//

    @Test
    public void test1Contains(){
        checkbook.add(debit);
        boolean expected = true;
        boolean actual = checkbook.contains(debit);
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void test2Contains(){
        checkbook.add(debit);
        boolean expected = false;
        boolean actual = checkbook.contains(credit);
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void test3Contains(){
        checkbook.add(credit);
        checkbook.add(debit);
        boolean expected = true;
        boolean actual = checkbook.contains(credit);
        boolean actual2 = checkbook.contains(debit);
        Assert.assertEquals(expected, actual);
        Assert.assertEquals(expected, actual2);
    }

    @Test
    public void test1Add(){
        checkbook.add(debit);
        int expected = 1;
        int actual = checkbook.size();
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void test2Add(){
        checkbook.add(debit);
        boolean expected = true;
        boolean actual = checkbook.contains(debit);
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void test3Add(){
        checkbook.add(credit);
        boolean expected = true;
        boolean actual = checkbook.contains(credit);
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void test1Remove(){
        checkbook.add(debit);
        checkbook.remove(debit);
        int expected = 0;
        int actual = checkbook.size();
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void test2Remove(){
        checkbook.add(debit);
        boolean expected = false;
        boolean actual = checkbook.remove(credit);
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void test3Remove(){
        checkbook.add(debit);
        boolean expected = true;
        boolean actual = checkbook.remove(debit);
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void test1Iterator(){
        checkbook.add(debit);
        checkbook.add(credit);
        String expected = "Itr";
        String actual = checkbook.iterator().getClass().getSimpleName();
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void test2Iterator(){
        checkbook.add(debit);
        String expected = "This is a Debit";
        String actual = checkbook.iterator().next().getMemo();
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void test3Iterator(){
        checkbook.add(debit);
        Payee expected = payeeDebit;
        Payee actual = checkbook.iterator().next().getPayee();
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void test1ToArray(){
        checkbook.add(debit);
        checkbook.add(credit);
        String expected = "Transaction[]";
        String actual = checkbook.toArray().getClass().getSimpleName();
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void test2ToArray(){
        checkbook.add(debit);
        double expected = 100.50;
        Transaction[] ret = checkbook.toArray();
        double actual = ret[0].getAmount();
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void test3ToArray(){
        Transaction[] expected = new Transaction[1];
        expected[0] = debit;
        Transaction[] actual = checkbook.toArray(expected);
        Assert.assertEquals(expected[0], actual[0]);
    }

    @Test
    public void test1CheckbookValue(){
        checkbook.add(debit);
        double expected = 100.50;
        double actual = checkbook.checkbookValue();
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void test2CheckbookValue(){
        checkbook.add(credit);
        double expected = -100.50;
        double actual = checkbook.checkbookValue();
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void test3CheckbookValue(){
        checkbook.add(credit);
        double expected = -100.50;
        double actual = checkbook.checkbookValue();
        Assert.assertEquals(expected, actual);
    }

//    @Test
//    public void test1GetTransactionsForDate(){
//        checkbook.add(credit);
//        checkbook.add(debit);
//        Transaction[] retarr = checkbook.getTransactionsForDate(LocalDate.now());
//        Transaction expected = credit;
//        Assert.assertEquals(expected, retarr[1]);
//    }

    @Test
    public void test1GetAllTransactionsForPayee(){
        checkbook.add(debit);
        checkbook.add(credit);
        checkbook.add(credit);
        checkbook.add(debit);
        Payee expected = debit.getPayee();
        Transaction[] arr = checkbook.getAllTransactionsForPayee(payeeDebit);
        Payee actual = arr[0].getPayee();
        Payee actual2 = arr[1].getPayee();
        Assert.assertEquals(expected, actual);
        Assert.assertEquals(expected, actual2);
    }

    @Test
    public void test2GetAllTransactionsForPayee(){
        checkbook.add(debit);
        checkbook.add(credit);
        checkbook.add(credit);
        Payee expected = credit.getPayee();
        Transaction[] arr = checkbook.getAllTransactionsForPayee(payeeCredit);
        Payee actual = arr[0].getPayee();
        Payee actual2 = arr[1].getPayee();
        Assert.assertEquals(expected, actual);
        Assert.assertEquals(expected, actual2);
    }

    @Test
    public void test3GetAllTransactionsForPayee(){
        checkbook.add(debit);
        checkbook.add(credit);
        Payee expected = credit.getPayee();
        Transaction[] arr = checkbook.getAllTransactionsForPayee(payeeCredit);
        Payee actual = arr[0].getPayee();
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void test1GetAllDebitTransactions(){
        checkbook.add(debit);
        checkbook.add(credit);
        Transaction expected = debit;
        Transaction[] arr = checkbook.getAllDebitTransactions();
        Transaction actual = arr[0];
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void test2GetAllDebitTransactions(){
        checkbook.add(debit);
        checkbook.add(credit);
        checkbook.add(debit);
        checkbook.add(debit);
        Transaction expected = debit;
        Transaction[] arr = checkbook.getAllDebitTransactions();
        Transaction actual = arr[0];
        Transaction actual2 = arr[1];
        Transaction actual3 = arr[2];
        Assert.assertEquals(expected, actual);
        Assert.assertEquals(expected, actual2);
        Assert.assertEquals(expected, actual3);
    }

    @Test
    public void test3GetAllDebitTransactions(){
        checkbook.add(credit);
        boolean expected = true;
        Transaction[] arr = checkbook.getAllDebitTransactions();
        boolean actual = arr.length == 0;
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void test1GetAllCreditTransactions(){
        checkbook.add(debit);
        checkbook.add(credit);
        Transaction expected = credit;
        Transaction[] arr = checkbook.getAllCreditTransactions();
        Transaction actual = arr[0];
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void test2GetAllCreditTransactions(){
        checkbook.add(debit);
        checkbook.add(credit);
        checkbook.add(debit);
        checkbook.add(debit);
        Transaction expected = credit;
        int expected2 = 1;
        Transaction[] arr = checkbook.getAllCreditTransactions();
        Transaction actual = arr[0];
        int actual2 = arr.length;
        Assert.assertEquals(expected, actual);
        Assert.assertEquals(expected2, actual2);
    }

    @Test
    public void test3GetAllCreditTransactions(){
        checkbook.add(debit);
        boolean expected = true;
        Transaction[] arr = checkbook.getAllCreditTransactions();
        boolean actual = arr.length == 0;
        Assert.assertEquals(expected, actual);
    }


    @org.junit.After
    public void tearDown() throws Exception {
        System.out.println("Running: TearDown");
        checkbook = null;
        assertNull(checkbook);
        if (checkbook == null) System.out.println("Checkbook null, TearDown complete.");
        else System.out.println("TearDown NOT successful, checkbook did not evaluate to NULL");
    }
}